# -*- coding: utf-8 -*-
from tempfile import mkstemp
import shutil
import os
import six
from netCDF4 import Dataset
import numpy as np

DIMS_ALIAS = {
    'map': ['latitude', 'lat', 'longitude', 'lon', 'i', 'j', 'x', 'y'],
    'map_ensemble': ['latitude', 'lat', 'longitude', 'lon', 'i', 'j', 'x', 'y',
                     'realization', 'ensemble', 'number'],
    'time': ['time'],
    'time_ensemble': ['time', 'realization', 'ensemble', 'number'],
}


def compress(input, output, deflate_level, shuffle, rename, chunkdims):
    if output is None:
        descriptor, temp = mkstemp(suffix='.nc')
        os.close(descriptor)
    else:
        temp = output

    if isinstance(chunkdims, six.string_types):
        if chunkdims.lower() in DIMS_ALIAS:
            chunkdims = DIMS_ALIAS[chunkdims.lower()]
        else:
            chunkdims = [chunkdims]
    original = Dataset(input, 'r')
    compressed = Dataset(temp, 'w')
    _copy_attributtes(compressed, original)
    _copy_dimensions(compressed, original, rename)
    for variable in original.variables.keys():
        _copy_variable(compressed, original, variable, deflate_level, shuffle,
                       rename, chunkdims)
    original.close()
    compressed.close()

    if output is None:
        shutil.move(temp, input)


def _copy_variable(new_file, original_file, variable, deflate_level, shuffle,
                   rename, chunkdims):
    print('Writing {}'.format(variable))
    original_var = original_file.variables[variable]
    if variable in rename:
        new_name = rename[variable]
    else:
        new_name = variable

    chunksizes = []
    dimensions = original_file.dimensions
    if len(dimensions) > 2:
        for dim in original_var.dimensions:
            if dim in chunkdims:
                chunksizes.append(dimensions[dim].size)
            else:
                chunksizes.append(1)
    else:
        chunksizes = [dimensions[dim].size for dim in original_var.dimensions]
    new_var = new_file.createVariable(
        new_name, original_var.datatype,
        _rename(original_var.dimensions, rename),
        zlib=True, complevel=deflate_level, shuffle=shuffle,
        chunksizes=chunksizes)

    attributtes = {
        k: convert_to_ascii_if_possible(original_var.getncattr(k))
        for k in original_var.ncattrs()
    }
    if 'coordinates' in attributtes:
        new_coords = []
        for coord in attributtes['coordinates'].split():
            if coord in rename:
                new_coords.append(rename[coord])
            else:
                new_coords.append(coord)
        attributtes['coordinates'] = convert_to_ascii_if_possible(
            ' '.join(new_coords))
    print(attributtes)
    new_var.setncatts(attributtes)
    new_file.sync()
    if len(original_var.shape) > 2:
        slices = []
        for dim in original_var.dimensions:
            if dim in chunkdims:
                slices.append(None)
            else:
                slices.append(dimensions[dim].size)
        slice_list = [slice(None) for x in range(len(slices))]
        ind = np.argmax(slice_list)
        for x in range(slices[ind]):
            print('Copying slice {} of {}'.format(x + 1, slices[ind]))
            slice_list[ind] = slice(x, x+1, 1)
            new_var[slice_list] = original_var[slice_list]
            new_file.sync()
    else:
        new_var[...] = original_var[...]
        new_file.sync()


def _rename(original, rename):
    return [rename[value] if value in rename else value for value in original]


def _copy_dimensions(new_file, original_file, rename):
    for dimension in original_file.dimensions.keys():
        if dimension in rename:
            new_name = rename[dimension]
        else:
            new_name = dimension
        new_file.createDimension(
            new_name,
            original_file.dimensions[dimension].size
        )
    new_file.sync()


def _copy_attributtes(new_file, original_file):
    for attribute in original_file.ncattrs():
        original = getattr(original_file, attribute)
        setattr(new_file, attribute, convert_to_ascii_if_possible(original))


def convert_to_ascii_if_possible(string, encoding='ascii'):
    u"""
    Convert an Unicode string to ASCII if all characters can be translated.

    If a string can not be translated it is unchanged. It also automatically
    replaces Bretonnière with Bretonniere

    Parameters
    ----------
    string: unicode
    encoding: str, optional

    Returns
    -------
    str

    """
    # noinspection PyTypeChecker
    if isinstance(string, six.string_types):
        try:
            return string.encode(encoding)
        except UnicodeEncodeError:
            pass
    return string
